import { Component, Input, OnInit } from '@angular/core';
import { ThemesService } from '@progress-chef/interact-themes';

@Component({
  selector: 'lib-clipboard',
  templateUrl: './clipboard.component.html',
  styleUrls: ['./clipboard.component.scss']
})
export class ClipboardComponent implements OnInit {

  @Input() clipboardData: string = "";
  @Input() tooltipPosition: 'top' | 'bottom' | 'left' | 'right' = 'top';
  @Input() preCopyText: string = "Copy";
  @Input() postCopyText: string = "Copied!";
  tooltipText: string = "";

  constructor(private themesService:ThemesService) { }

  ngOnInit() {
    this.tooltipText = this.preCopyText;
  }

  async copyToClipboard() {
    if (!this.clipboardData || this.clipboardData.trim() === '') {
      throw new Error('Text to copy is empty or invalid.');
    }
    await this.copy(this.clipboardData);
    this.tooltipText = this.postCopyText;
  }

  /**
   * Asynchronously copies text to the clipboard.
   * @param clipboardData The text to be copied to the clipboard.
   */
  async copy(clipboardData: string) {
    // Check if the modern Clipboard API is supported in the browser
    if (window.navigator.clipboard) {
      // If supported, use the modern copy method
      await this.copyModern(clipboardData);
      return;
    }

    // If not supported, fallback to the legacy copy method
    await this.copyLegacy(clipboardData);
  }

  /**
   * Asynchronously copies text to the clipboard using the modern Clipboard API.
   * @param clipboardData The text to be copied to the clipboard.
   */
  async copyModern(clipboardData: string) {
    try {
      // Attempt to copy using the modern Clipboard API
      await window.navigator.clipboard.writeText(clipboardData);
    } catch (err) {
      // Handle any errors that occur during the copy operation
      console.log("Error using the modern Clipboard API[window.navigator.clipboard.writeText]:\n", err);
      // If there's an error, fall back to the legacy copy method
      await this.copyLegacy(clipboardData);
    }
  }

  /**
   * Asynchronously copies text to the clipboard using a legacy method.
   * @param clipboardData The text to be copied to the clipboard.
   * @returns A Promise that resolves with a boolean indicating success.
   */
  async copyLegacy(clipboardData: string): Promise<boolean> {
    return new Promise<boolean>((resolve, reject) => {
      try {
        // Create a hidden textarea element off-screen
        const el = document.createElement('textarea');
        el.style.position = 'fixed';
        el.style.left = '-9999px';
        el.value = clipboardData;

        // Append the element to the document body
        document.body.appendChild(el);

        // Select the text within the textarea and attempt to copy
        el.select();
        const copied = document.execCommand('copy');

        // Clean up by removing the textarea element
        document.body.removeChild(el);

        // Resolve the Promise with a boolean indicating success
        resolve(copied);
      } catch (err) {
        // If there's an error during the legacy copy operation, reject the Promise
        reject(err);
      }
    });
  }

  // Once mouse leave the action button reverting the tooltip to original state.
  revertTooltipText() {
    this.tooltipText = this.preCopyText;
  }

}
