import {
  moduleMetadata,
  Meta,
  StoryObj,
  applicationConfig,
} from '@storybook/angular';
import { DropDownsModule } from '@progress/kendo-angular-dropdowns';
import { DropdowntreeComponent } from '@components/atoms/dropdowntree/dropdowntree.component';
import { FormControl, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { TypographyModule } from '@components/atoms/typography/typography.module';
import { ButtonModule } from '@components/atoms/button/button.module';
import '@angular/localize/init';

import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { userEvent, within } from '@storybook/test';
import { provideAnimations } from '@angular/platform-browser/animations';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<DropdowntreeComponent> = {
  title: 'Atoms/DropdownTree Component',
  component: DropdowntreeComponent,
  decorators: [
    applicationConfig({
      providers: [provideAnimations()],
    }),
    moduleMetadata({
      imports: [
        DropDownsModule,
        ReactiveFormsModule,
        TypographyModule,
        ThemesModule,
        FormsModule,
        ButtonModule,
      ],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    data: {
      description: `Sets the data of the ComboBox. The data has to be provided in an array-like list.`,
    },
    isDisabled: {
      description: `Sets the disabled state of the component.`,
      table: {
        defaultValue: {
          summary: 'false',
        },
      },
    },
    clearButton: {
      description: `If set to true, renders a button on hovering over the component. Clicking this button resets the value of the component to undefined.`,
      table: {
        defaultValue: {
          summary: 'false',
        },
      },
    },
    fillMode: {
      description: 'Sets the fillMode of the component.',
      options: ['flat', 'solid', 'outline', 'none'],
      control: 'radio',
      table: {
        defaultValue: {
          summary: 'solid',
        },
      },
    },
    placeholder: {
      description: `The hint that is displayed when the component is empty.`,
      control: 'text',
      table: {
        defaultValue: {
          summary: '',
        },
      },
    },
    readonly: {
      description: `Sets the read-only state of the component.`,
      table: {
        defaultValue: {
          summary: 'false',
        },
      },
    },
    rounded: {
      description: 'Sets the border radius of the component.',
      options: ['small', 'medium', 'large', 'full', 'none'],
      control: 'radio',
      table: {
        defaultValue: {
          summary: 'medium',
        },
      },
    },
    size: {
      description: 'Sets the size of the component.',
      options: ['small', 'medium', 'large', 'none'],
      control: 'radio',
      table: {
        defaultValue: {
          summary: 'medium',
        },
      },
    },
    textField: {
      description: `Sets the data item field that represents the item text. If the data contains only primitive values, do not define it. The textField property can be set to point to a nested property value - e.g. category.name.`,
      control: 'text',
    },
    value: {
      description: `Sets the value of the DropDownTree. It can either be of the primitive (string, numbers) or of the complex (objects) type. To define the type, use the valuePrimitive option.`,
    },
    valueField: {
      description: `The fields of the data item that provide the value of the nodes inside the DropDownTree. The valueField property can be set to point to a nested property value - e.g. category.id.`,
      control: 'text',
    },
    listHeight: {
      description: `Sets the height of the options list in the popup. By default, listHeight is 200px.`,
      control: 'number',
    },
    hasDefaultValue: {
      description: `If set to true, the input value will be selected by default`,
      control: 'boolean',
    },
    footerBtnLabel: {
      description: `Footer button name`,
      control: 'text',
    },
    footerBtnIcon: {
      description: `Icon used in footer button`,
      control: 'text',
    },
    hasFooterBtn: {
      control: 'boolean',
    },
    control: {
      description: `FormControl to be used in reactive forms. Example: [control]="parentForm.get('form-control-name)"`,
    },
  },
  render: (args: DropdowntreeComponent) => ({
    props: args,
    template: `
    <lib-dropdowntree 
      [data]=data [placeholder]=placeholder [isDisabled]="isDisabled" [textField]="textField"
      [value]="value" [valueField]="valueField" [expandBy]="expandBy" [expandedKeys]="expandedKeys"
      [childrenField]="childrenField" [fillMode]="fillMode" [rounded]="rounded" [size]="size"
      [clearButton]="clearButton" [readonly]="readonly" [hasDefaultValue]="hasDefaultValue" 
      [footerBtnLabel]="footerBtnLabel" [footerBtnIcon]="footerBtnIcon" [hasFooterBtn]="hasFooterBtn">
  </lib-dropdowntree>`,
  }),
};

export default meta;

type Story = StoryObj<DropdowntreeComponent>;

export const DropdownTree: Story = {
  args: {
    value: null,
    clearButton: false,
    expandBy: 'id',
    expandedKeys: [1, 5],
    data: [
      {
        id: 1,
        text: 'Cloud Integrations',
        items: [
          {
            id: 2,
            text: 'Azure',
            imageUrl: 'https://swimburger.net/media/ppnn3pcl/azure.png',
          },
          {
            id: 3,
            text: 'Amazon Web Services',
            imageUrl:
              'https://upload.wikimedia.org/wikipedia/commons/thumb/9/93/Amazon_Web_Services_Logo.svg/1280px-Amazon_Web_Services_Logo.svg.png',
          },
          {
            id: 4,
            text: 'Google Cloud Platform',
            imageUrl:
              'https://1000logos.net/wp-content/uploads/2020/05/Logo-Google-Cloud.jpg',
          },
        ],
      },
      {
        id: 5,
        text: 'Container Integrations',
        items: [
          {
            id: 6,
            text: 'Kubernetes',
            imageUrl:
              'https://upload.wikimedia.org/wikipedia/commons/3/39/Kubernetes_logo_without_workmark.svg',
          },
          {
            id: 7,
            text: 'Docker',
            imageUrl:
              'https://upload.wikimedia.org/wikipedia/commons/7/79/Docker_%28container_engine%29_logo.png',
          },
          {
            id: 8,
            text: 'Openshift',
            imageUrl:
              'https://upload.wikimedia.org/wikipedia/commons/3/3a/OpenShift-LogoType.svg',
          },
        ],
      },
    ],
    isDisabled: false,
    childrenField: 'items',
    textField: 'text',
    valueField: 'id',
    placeholder: 'Select...',
    fillMode: 'solid',
    rounded: 'medium',
    size: 'medium',
    readonly: false,
    listHeight: 330,
    footerBtnLabel: 'Manual Integration',
    footerBtnIcon: 'add',
    hasDefaultValue: false,
    hasFooterBtn: true,
    control: new FormControl(),
  },
  play: async ({ canvasElement }) => {
    const canvas = within(canvasElement);
    await userEvent.click(canvas.getByRole('combobox'));
  },
};
