import {
  moduleMetadata,
  Meta,
  StoryObj,
  applicationConfig,
} from '@storybook/angular';
import { DropDownsModule } from '@progress/kendo-angular-dropdowns';
import { MultiSelectDropdownComponent } from '@components/atoms/multi-select-dropdown/multi-select-dropdown.component';
import { FormControl, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { TypographyModule } from '@components/atoms/typography/typography.module';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import '@angular/localize/init';
import { provideAnimations } from '@angular/platform-browser/animations';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<MultiSelectDropdownComponent> = {
  title: 'Atoms/MultiSelect Dropdown Component',
  component: MultiSelectDropdownComponent,
  decorators: [
    applicationConfig({
      providers: [provideAnimations()],
    }),
    moduleMetadata({
      imports: [
        DropDownsModule,
        ReactiveFormsModule,
        TypographyModule,
        ThemesModule,
        FormsModule,
      ],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    id: {
      description: `Id to be assigned to the input element.`,
      control: 'text',
      type: {
        required: true,
        name: 'string',
      },
    },
    label: {
      description: `Text to be shown in the label. If this is empty string, no label will be shown.`,
      control: 'text',
      type: {
        required: true,
        name: 'string',
      },
    },
    fontType: {
      description: 'Type of the font family.',
      control: 'radio',
      options: [
        // h1 styles
        'H1_MULISH_36_BOLD',
        'H1_MULISH_36_REGULAR',
        'H1_MULISH_36_LIGHT',
        // h2 styles
        'H2_MULISH_24_BOLD',
        'H2_MULISH_24_REGULAR',
        'H2_MULISH_24_LIGHT',
        // h3 styles
        'H3_MULISH_18_BOLD',
        'H3_MULISH_18_REGULAR',
        'H3_MULISH_18_LIGHT',
        // h4, h5 and h6 styles
        'H4_MULISH_14_BOLD',
        'H5_MULISH_14_REGULAR',
        'H6_MULISH_14_LIGHT',
        // p 16px styles
        'P_MULISH_16_BOLD',
        'P_MULISH_16_REGULAR',
        'P_MULISH_16_REGULAR_ITALIC',
        'P_MULISH_16_LIGHT',
        'P_MULISH_16_LIGHT_ITALIC',
        // p 14px styles
        'P_MULISH_14_BOLD',
        'P_MULISH_14_REGULAR',
        'P_MULISH_14_REGULAR_ITALIC',
        'P_MULISH_14_LIGHT',
        'P_MULISH_14_LIGHT_ITALIC',
        // small 12px styles
        'SMALL_MULISH_12_BOLD',
        'SMALL_MULISH_12_REGULAR',
        'SMALL_MULISH_12_REGULAR_ITALIC',
        'SMALL_MULISH_12_LIGHT',
        'SMALL_MULISH_12_LIGHT_ITALIC',
        // Custom
        'BASE_TEXT_MULISH',
      ],
      table: {
        defaultValue: {
          summary: 'P_MULISH_14_REGULAR',
        },
      },
    },
    data: {
      description: `Sets the data of the ComboBox. The data has to be provided in an array-like list.`,
    },
    isDisabled: {
      description: `Sets the disabled state of the component.`,
      table: {
        defaultValue: {
          summary: 'false',
        },
      },
    },
    clearButton: {
      description: `If set to true, renders a button on hovering over the component. Clicking this button resets the value of the component to undefined.`,
      table: {
        defaultValue: {
          summary: 'false',
        },
      },
    },
    fillMode: {
      description: 'Sets the fillMode of the component.',
      options: ['flat', 'solid', 'outline', 'none'],
      control: 'radio',
      table: {
        defaultValue: {
          summary: 'solid',
        },
      },
    },
    filterable: {
      description: `Enables the filtering functionality.`,
      table: {
        defaultValue: {
          summary: 'true',
        },
      },
    },
    placeholder: {
      description: `The hint that is displayed when the component is empty.`,
      control: 'text',
      table: {
        defaultValue: {
          summary: '',
        },
      },
    },
    readonly: {
      description: `Sets the read-only state of the component.`,
      table: {
        defaultValue: {
          summary: 'false',
        },
      },
    },
    rounded: {
      description: 'Sets the border radius of the component.',
      options: ['small', 'medium', 'large', 'full', 'none'],
      control: 'radio',
      table: {
        defaultValue: {
          summary: 'medium',
        },
      },
    },
    size: {
      description: 'Sets the size of the component.',
      options: ['small', 'medium', 'large', 'none'],
      control: 'radio',
      table: {
        defaultValue: {
          summary: 'medium',
        },
      },
    },
    textField: {
      description: `Sets the data item field that represents the item text. If the data contains only primitive values, do not define it. The textField property can be set to point to a nested property value - e.g. category.name.`,
      control: 'text',
    },
    valueField: {
      description: `Sets the data item field that represents the item value. If the data contains only primitive values, do not define it. The valueField property can be set to point to a nested property value - e.g. category.id.`,
      control: 'text',
    },
    valuePrimitive: {
      description: `Specifies the type of the selected value. If set to true, the selected value has to be of the primitive type.`,
      table: {
        defaultValue: {
          summary: 'true',
        },
      },
    },
    model: {
      description: `Value to be sent inside [(ngModel)] for template driven forms, don't use it with control field. Example1: [(model)]="model", Example2: (model)="model" (modelChange)="modelChange($event)"`,
    },
    control: {
      description: `FormControl to be used in reactive forms, don't use it with model field. Example: [control]="parentForm.get('form-control-name)"`,
    },
    emitModelChange: {
      action: true,
    },
    focus: {
      options: [true, false],
      control: 'radio',
    },
    autoClose: {
      options: [true, false],
      control: 'radio',
    },
  },
  render: (args: MultiSelectDropdownComponent) => ({
    props: args,
    template: `
      <lib-multi-select-dropdown [autoClose]="autoClose" [checkboxes]="checkboxes" [textField]="textField" [valueField]="valueField" [data]="data" [(model)]="model" [valuePrimitive]="valuePrimitive" [id]="id" [label]="label" [focus]="focus" [isDisabled]="isDisabled" [clearButton]="clearButton" [fillMode]="fillMode" [filterable]="filterable" [placeholder]="placeholder" [readonly]="readonly" [rounded]="rounded" [size]="size" [fontType]="fontType">
      </lib-multi-select-dropdown>
    `,
  }),
};

export default meta;
type Story = StoryObj<MultiSelectDropdownComponent>;

export const Dropdown: Story = {
  args: {
    autoClose: false,
    checkboxes: true,
    id: 'itemId',
    label: 'Item selection',
    data: [
      {
        id: 1,
        name: 'Azure',
      },
      {
        id: 2,
        name: 'Amazon web services',
      },
      {
        id: 3,
        name: 'Google Cloud Platform',
      },
      {
        id: 4,
        name: 'GCP',
      },
    ],
    focus: false,
    fillMode: 'solid',
    rounded: 'medium',
    size: 'medium',
    clearButton: true,
    readonly: false,
    isDisabled: false,
    fontType: 'P_MULISH_16_REGULAR',
    filterable: true,
    placeholder: 'Select appropriate item',
    textField: 'name',
    valueField: 'id',
    valuePrimitive: true,
    model: [3],
    control: new FormControl(),
  },
};
