import {
  moduleMetadata,
  StoryObj,
  Meta,
  applicationConfig,
} from '@storybook/angular';
import { FormControl, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { userEvent, within } from '@storybook/test';
import { NumericTextboxComponent } from '@components/atoms/numeric-textbox/numeric-textbox.component';
import { InputsModule } from '@progress/kendo-angular-inputs';
import { provideAnimations } from '@angular/platform-browser/animations';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<NumericTextboxComponent> = {
  title: 'Atoms/Numeric Textbox Component',
  component: NumericTextboxComponent,
  decorators: [
    applicationConfig({
      providers: [provideAnimations()],
    }),
    moduleMetadata({
      imports: [InputsModule, ReactiveFormsModule, ThemesModule, FormsModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    id: {
      description: `Id to be assigned to the input element.`,
      control: 'text',
      type: {
        required: true,
        name: 'string',
      },
    },
    autoCorrect: {
      description: `Specifies whether the value will be auto-corrected based on the minimum and maximum values.`,
      table: {
        defaultValue: {
          summary: true,
        },
      },
    },
    width: {
      description: `Width of numeric textbox in pixels.`,
      control: 'number',
      table: {
        defaultValue: {
          summary: 200,
        },
      },
    },
    isDisabled: {
      description: `Sets the disabled state of the component.`,
      table: {
        defaultValue: {
          summary: false,
        },
      },
    },
    value: {
      description: `Specifies the value of the NumericTextBox.`,
      control: 'number',
    },
    format: {
      description: `Specifies the format of value in the NumericTextBox. Refer for more details: https://github.com/telerik/kendo-intl/blob/master/docs/num-formatting/index.md`,
      control: 'text',
    },
    min: {
      description: `Specifies the smallest value that is valid.`,
      control: 'number',
    },
    max: {
      description: `Specifies the greatest value that is valid.`,
      control: 'number',
    },
    step: {
      description: `Specifies the value that is used to increment or decrement the component value.`,
      control: 'number',
      table: {
        defaultValue: {
          summary: 1,
        },
      },
    },
    fillMode: {
      description:
        'The fillMode property specifies the background and border styles of the NumericTextBox.',
      options: ['flat', 'solid', 'outline', 'none'],
      control: 'radio',
      table: {
        defaultValue: {
          summary: 'solid',
        },
      },
    },
    rounded: {
      description:
        'The rounded property specifies the border radius of the NumericTextBox.',
      options: ['small', 'medium', 'large', 'full', 'none'],
      control: 'radio',
      table: {
        defaultValue: {
          summary: 'medium',
        },
      },
    },
    size: {
      description:
        'The size property specifies padding of the NumericTextBox internal input element.',
      options: ['small', 'medium', 'large', 'none'],
      control: 'radio',
      table: {
        defaultValue: {
          summary: 'medium',
        },
      },
    },
    readonly: {
      description: `Determines whether the NumericTextBox is in its read-only state.`,
      table: {
        defaultValue: {
          summary: false,
        },
      },
    },
    model: {
      description: `Value to be sent inside [(ngModel)] for template driven forms, don't use it with control field. Example1: [(model)]="model", Example2: (model)="model" (modelChange)="modelChange($event)"`,
    },
    control: {
      description: `FormControl to be used in reactive forms, don't use it with model field. Example: [control]="parentForm.get('form-control-name)"`,
    },
    emitModelChange: {
      action: true,
    },
    placeholder: {
      control: 'text',
    },
  },
  render: (args: NumericTextboxComponent) => ({
    props: args,
  }),
};

export default meta;

type Story = StoryObj<NumericTextboxComponent>;

export const NumericTextbox: Story = {
  args: {
    id: 'itemId',
    autoCorrect: true,
    width: 200,
    isDisabled: false,
    format: 'n0',
    value: 10,
    placeholder: '--',
    min: 1,
    max: 20,
    step: 2,
    fillMode: 'solid',
    rounded: 'medium',
    size: 'medium',
    readonly: false,
    model: 2,
    control: new FormControl(),
  },
  play: async ({ canvasElement }) => {
    const canvas = within(canvasElement);
    await userEvent.click(canvas.getByRole('spinbutton'));
  },
};
